
#ifndef GRAPH_WIDGET_HH_
#define GRAPH_WIDGET_HH_

#include <QString>
#include <QSet>
#include <QHash>
#include <QtGui/QGraphicsView>

class node;
class edge;

class graph_widget : public QGraphicsView
{
	Q_OBJECT

public:
	graph_widget(QWidget* parent = 0);
	virtual ~graph_widget();

    void itemMoved();

public slots:
	void add_node(const QString& name);
	void remove_node(const QString& name);
	void add_connection(const QString& key, const QString& src, 
						const QString& dst, const QString& why);
	void remove_connection(const QString& key);

protected:
    void timerEvent(QTimerEvent *event);
    void wheelEvent(QWheelEvent *event);

    void scaleView(qreal scaleFactor);

private:
    int timerId;

	/* a table between agent name and their representation */
	QHash<QString, node*> node_table;

	/* a table between request src_dst and their associated edge, make sure
	 * there is 0 or 1 directed edge between two nodes */
	QHash<QString, edge*> edge_key_table;

	/*a table between request identifiant (caller, id)  and the associated edge,
	 * multiples identifiers can match the same edge
	 */
	QHash<QString, edge*> edge_table;

	/* a table between request identifiant (caller, id) and the associated msg
	 * related to this request */
	QHash<QString, QString> reason_table;

	/*
	 * Store the key connection not found. In certain case, we receive the
	 * destruction of the connection between its construction. To avoid this
	 * buggy case, store the removed connection not found, and when ask for an
	 * insertion, check in this set if it is not already destroyed
	 */
	QSet<QString> removed_connection;
};

#endif /* GRAPH_WIDGET_HH_ */
