#ifndef HYPER_REPLAY_NODE_HH_
#define HYPER_REPLAY_NODE_HH_

#include <QString>
#include <QMutex>
#include <QWaitCondition>
#include <deque>

#include <hyper_node.hh>
#include <boost/variant.hpp>
#include <boost/date_time/posix_time/posix_time.hpp>

namespace details {
	struct add_link {
		QString src;
		QString id;
		QString dst;
		QString content;

		add_link() {}
		add_link(const QString& src, const QString& id, const QString& dst,
				 const QString& content) :
			src(src), id(id), dst(dst), content(content)
		{}
	};

	struct remove_link {
		QString src;
		QString id;
		size_t reverse;

		remove_link() {}
		remove_link(const QString& src, const QString& id) :
			src(src), id(id)
		{}
	};

	struct new_agent {
		QString agent;

		new_agent() {}
		new_agent(const QString& agent) : agent(agent) {}
	};

	struct remove_agent {
		QString agent;

		remove_agent() {}
		remove_agent(const QString& agent) : agent(agent) {}
	};

	struct event {
		typedef boost::variant<
			add_link,
			remove_link,
			new_agent, 
			remove_agent
		> type;

		type ev;
		boost::posix_time::ptime abs_time;
		boost::posix_time::time_duration rel_time; /* since previous */
	};
}

class hyper_replay_node : public hyper_node
{
	Q_OBJECT

	public:
		hyper_replay_node(const std::string& s, double slowing_factor = 1.0);
		~hyper_replay_node();
		virtual void run();
		void pause();
		void restart();
		void move(int i);

		size_t size() const { return events.size(); }
	
	private:
		std::deque<details::event> events;
		bool must_pause;
		QWaitCondition waitRestart;
		QMutex mutex;

		size_t current_idx;

		double slowing_factor;

		friend struct send_inverse_event;
};

#endif /* HYPER_REPLAY_NODE_HH_ */
