#include <replay_widget.hh>
#include <graph_widget.hh>
#include <iostream>

#include <QFont>
#include <QPushButton>
#include <QStyle>
#include <QToolButton>
#include <QApplication>
#include <QBoxLayout>
#include <QFileDialog>

ReplayWidget::ReplayWidget(QWidget* parent) : QWidget(parent), s(IDLE), 
	slider_user_interaction(false), 
	node(0)
{
	init();
}

ReplayWidget::ReplayWidget(const std::string& filename, QWidget* parent) : QWidget(parent), s(IDLE),
	slider_user_interaction(false),
	node(0)
{
	init();
	open_log(filename);
}

void ReplayWidget::init()
{
//	setFixedSize(200, 120);

	graph = new graph_widget();

	QSize iconSize(36, 36);

	QToolButton* openButton = new QToolButton;
	openButton->setIcon(style()->standardIcon(QStyle::SP_DialogOpenButton));
	openButton->setIconSize(iconSize);
	openButton->setToolTip(tr("Open File"));
	connect(openButton, SIGNAL(clicked()), this, SLOT(open()));

	playButton = new QToolButton;
	playButton->setIcon(style()->standardIcon(QStyle::SP_MediaPlay));
	playButton->setIconSize(iconSize);
	playButton->setToolTip(tr("Play"));
	connect(playButton, SIGNAL(clicked()), this, SLOT(start()));

	pauseButton = new QToolButton;
	pauseButton->setCheckable(true);
	pauseButton->setIcon(style()->standardIcon(QStyle::SP_MediaPause));
	pauseButton->setIconSize(iconSize);
	pauseButton->setToolTip(tr("Pause"));
	connect(pauseButton, SIGNAL(clicked(bool)), this, SLOT(pause(bool)));

	QToolButton* stopButton = new QToolButton;
	stopButton->setIcon(style()->standardIcon(QStyle::SP_MediaStop));
	stopButton->setIconSize(iconSize);
	stopButton->setToolTip(tr("Stop"));
	//connect(stopButton, SIGNAL(clicked()), movie, SLOT(stop()));

	QToolButton* quitButton = new QToolButton;
	quitButton->setIcon(style()->standardIcon(QStyle::SP_DialogCloseButton));
	quitButton->setIconSize(iconSize);
	quitButton->setToolTip(tr("Quit"));
	connect(quitButton, SIGNAL(clicked()), qApp, SLOT(quit()));

	event_slider = new QSlider(Qt::Horizontal);

	connect(event_slider, SIGNAL(sliderPressed()), this, SLOT(sliderPressed()));
	connect(event_slider, SIGNAL(sliderReleased()), this, SLOT(sliderReleased()));
	connect(event_slider, SIGNAL(valueChanged(int)), this, SLOT(sliderValueChanged(int)));

	QHBoxLayout* buttonsLayout = new QHBoxLayout;
	buttonsLayout->addStretch();
	buttonsLayout->addWidget(openButton);
	buttonsLayout->addWidget(playButton);
	buttonsLayout->addWidget(pauseButton);
	buttonsLayout->addWidget(stopButton);
	buttonsLayout->addWidget(quitButton);
	buttonsLayout->addWidget(event_slider);
	buttonsLayout->addStretch();


	QBoxLayout *layout = new QBoxLayout(QBoxLayout::TopToBottom, this);
	layout->addWidget(graph);
	layout->addLayout(buttonsLayout);


	update_buttons();
}

void ReplayWidget::start()
{
	QObject::connect(node, SIGNAL(new_agent(const QString&)),
					 graph, SLOT(add_node(const QString&)));
	QObject::connect(node, SIGNAL(remove_agent(const QString&)),
					 graph, SLOT(remove_node(const QString&)));
	QObject::connect(node, SIGNAL(add_link(const QString&, const QString&, const QString&, const QString&)),
					 graph, SLOT(add_connection(const QString&, const QString&, const QString&, const QString&)));
	QObject::connect(node, SIGNAL(remove_link(const QString&)),
					 graph, SLOT(remove_connection(const QString&)));

	QObject::connect(node, SIGNAL(new_agent(const QString&)),
					 this , SLOT(new_event(const QString&)));
	QObject::connect(node, SIGNAL(remove_agent(const QString&)),
					 this, SLOT(new_event(const QString&)));
	QObject::connect(node, SIGNAL(add_link(const QString&, const QString&, const QString&, const QString&)),
					 this, SLOT(new_event(const QString&, const QString&, const QString&, const QString&)));
	QObject::connect(node, SIGNAL(remove_link(const QString&)),
					 this, SLOT(new_event(const QString&)));

	if (s == IDLE) 
		node->start();

	if (s == PAUSE) 
		node->restart();

	s = PLAY;
	update_buttons();
}

void ReplayWidget::open()
{
	QString fileName = QFileDialog::getOpenFileName(this, tr("Open a log file"));

	if (node)
		delete node;

     if (!fileName.isEmpty())
		open_log(qPrintable(fileName));
}

void ReplayWidget::open_log(const std::string& s) {
	node = new hyper_replay_node(s);

	event_slider->setRange(0, node->size());
}

void ReplayWidget::pause(bool b)
{
	s = PAUSE;
	node->pause();
	update_buttons();
}

void ReplayWidget::update_buttons()
{
	playButton->setEnabled(s == IDLE || s == PAUSE);
	pauseButton->setEnabled(s == PLAY);
}

void ReplayWidget::new_event(const QString&)
{
	if (!slider_user_interaction)
		event_slider->setValue(event_slider->value() + 1);
}

void ReplayWidget::new_event(const QString& s, const QString&, const QString&, const QString&)
{
	new_event(s);
}

void ReplayWidget::sliderPressed() {
	slider_user_interaction = true;
	if (s == PLAY)
		node->pause();
}

void ReplayWidget::sliderReleased() {
	slider_user_interaction = false;
	if (s == PLAY)
		node->restart();
}

void ReplayWidget::sliderValueChanged(int i) {
	if (slider_user_interaction) 
		node->move(i);
}
